using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Telerik.WinControls;
using Telerik.WinControls.UI;
using Telerik.WinControls.Enumerations;
using Telerik.WinControls.Layout;

namespace MultiSelectComboBox
{
	public delegate void SelectedItemsChangedEventHandler(object sender, ItemsChangedEventArgs e);

	public class ItemsChangedEventArgs : EventArgs
    {
        private ItemsChangeOperation m_operation;
        private RadComboBoxItem m_newValue;

        public ItemsChangedEventArgs(RadComboBoxItem p_newValue, ItemsChangeOperation p_operation)
        {
            m_newValue = p_newValue;
            m_operation = p_operation;
        }

        public RadComboBoxItem NewValue
        {
            get
            {
                return m_newValue;
            }
        }

        public ItemsChangeOperation Operation
        {
            get
            {
                return m_operation;
            }
        }
    }

    public class RadCustomComboBoxItem : RadComboBoxItem
    {
        public RadCustomComboBoxItem()
        {
        }

        public RadCustomComboBoxItem(string text)
            : base(text)
        {
        }

        public RadCustomComboBoxItem(string text, MouseEventHandler OnClick)
            : base(text, OnClick)
        {
        }

        private RadCheckBoxElement chBox;

        public RadCheckBoxElement CheckBox
        {
            get
            {
                EnsureChildElements();
                return chBox;
            }
        }

        protected override void CreateChildElements()
        {
            base.CreateChildElements();

            // add a checkbox to the current item
            this.Children[2].Children[1].Margin = new Padding(15, 0, 0, 0);
            this.chBox = new RadCheckBoxElement();
            chBox.StretchHorizontally = false;
            chBox.StretchVertically = false;
            chBox.ToggleStateChanged += new StateChangedEventHandler(chBox_ToggleStateChanged);
            chBox.Click += new EventHandler(chBox_Click);
            this.Children.Add(chBox);
        }

        void chBox_Click(object sender, EventArgs e)
        {
            // if a checkbox is clicked, this means that the combo is updated from the UI            
            RadGridView gridControl = (RadGridView)this.OwnerElement.ElementTree.Control;
            BaseGridEditor editor = (BaseGridEditor)gridControl.ActiveEditor;
            CustomComboBoxEditorElement editorElement = (CustomComboBoxEditorElement)editor.EditorElement;
            editorElement.updatedFromUI = true;
        }

        public RadCheckBoxElement GetAssociatedCheckBox()
        {
            return (RadCheckBoxElement)this.Children[3];
        }

        void chBox_ToggleStateChanged(object sender, StateChangedEventArgs args)
        {
            RadGridView gridControl = (RadGridView)this.OwnerElement.ElementTree.Control;
            if (gridControl != null)
            {
                BaseGridEditor editor = (BaseGridEditor)gridControl.ActiveEditor;
                CustomComboBoxEditorElement editorElement = (CustomComboBoxEditorElement)editor.EditorElement;
                // if the combo is updated from the UI, we add/remove the current item
                if (editorElement.updatedFromUI)
                {
                    if (chBox.IsChecked)
                    {
                        editorElement.SelectedItems.Add(this);
                    }
                    else
                    {
                        editorElement.SelectedItems.Remove(this);
                    }
                }
            }
        }
    }

    public class CustomComboBoxEditor : RadComboBoxEditor
    {
        public override object Value
        {
            get
            {
                CustomComboBoxEditorElement editorElement = (CustomComboBoxEditorElement)this.EditorElement;
                return editorElement.Text;                
            }
            set
            {
                CustomComboBoxEditorElement editorElement = (CustomComboBoxEditorElement)this.EditorElement;
                if (value != null && value != DBNull.Value)
                {
                    string[] selectedItems = value.ToString().Split(';');
                    foreach (string token in selectedItems)
                    {
                        string parsedToken = token.TrimStart();
                        foreach (RadCustomComboBoxItem item in editorElement.Items)
                        {
                            if (parsedToken == item.Text)
                            {
                                editorElement.SelectedItems.Add(item);
                                break;
                            }
                        }
                    }
                }
            }
        }

        public override void Initialize(object owner, object value)
        {
            CustomComboBoxEditorElement editorElement = (CustomComboBoxEditorElement)this.EditorElement;
            editorElement.DataSource = null;
            editorElement.Items.Clear();
            editorElement.SelectedItems.Clear();
            for (int i = 0; i < 5; i++)
            {
                editorElement.Items.Add(new RadCustomComboBoxItem("item " + i));
            }
            base.Initialize(owner, value);
        }

        protected override RadElement CreateEditorElement()
        {
            return new CustomComboBoxEditorElement();
        }
    }

    public class CustomComboBoxEditorElement : RadComboBoxEditorElement
    {
        RadListBoxItemCollection selectedItems;
        internal bool updatedFromUI = false;

        public CustomComboBoxEditorElement()
        {
            selectedItems = new RadListBoxItemCollection();
            selectedItems.CollectionChanged += new RadListBoxItemCollectionChangedEventHandler(selectedItems_CollectionChanged);
            this.PopupClosed += new RadPopupClosedEventHandler(CustomComboBoxEditorElement_PopupClosed);
        }

        public RadListBoxItemCollection SelectedItems
        {
            get
            {
                return selectedItems;
            }
        }

        public event SelectedItemsChangedEventHandler SelectedItemsChanged;

        protected virtual void OnSelectedItemsChanged(ItemsChangedEventArgs args)
        {
            if (SelectedItemsChanged != null)
            {
                SelectedItemsChanged(this, args);
            }
        }

        private void selectedItems_CollectionChanged(object sender, RadListBoxItemCollectionChangedEventArgs e)
        {
            RadCustomComboBoxItem customComboItem = null;
            if (e.NewValue != null)
            {
                customComboItem = e.NewValue as RadCustomComboBoxItem;
            }
            else if (e.OldValue != null)
            {
                customComboItem = e.OldValue as RadCustomComboBoxItem;
            }

            if (e.Operation == ItemsChangeOperation.Removed)
            {
                if (!updatedFromUI)
                {
                    customComboItem.GetAssociatedCheckBox().IsChecked = false;
                }
            }
            else if (e.Operation == ItemsChangeOperation.Inserted)
            {
                if (!updatedFromUI)
                {
                    customComboItem.GetAssociatedCheckBox().IsChecked = true;
                }
            }
            else if (e.Operation == ItemsChangeOperation.Clearing)
            {
                foreach (RadCustomComboBoxItem selectedItem in this.SelectedItems)
                {
                    selectedItem.GetAssociatedCheckBox().IsChecked = false;
                }
            }

            updatedFromUI = false;
            customComboItem = null;

            SetText();

            if (e.Operation == ItemsChangeOperation.Inserted)
            {
                ItemsChangedEventArgs args = new ItemsChangedEventArgs((RadComboBoxItem)e.NewValue, e.Operation);
                this.OnSelectedItemsChanged(args);
            }
            else if (e.Operation == ItemsChangeOperation.Removed)
            {
                ItemsChangedEventArgs args = new ItemsChangedEventArgs((RadComboBoxItem)e.OldValue, e.Operation);
                this.OnSelectedItemsChanged(args);
            }
            else if (e.Operation == ItemsChangeOperation.Cleared)
            {
                ItemsChangedEventArgs args = new ItemsChangedEventArgs((RadComboBoxItem)e.NewValue, e.Operation);
                this.OnSelectedItemsChanged(args);
            }
        }

        private void CustomComboBoxEditorElement_PopupClosed(object sender, RadPopupClosedEventArgs args)
        {
            this.SelectedItem = null;
            SetText();
        }

        private void SetText()
        {
            this.Text = String.Empty;
            foreach (RadCustomComboBoxItem selectedItem in this.SelectedItems)
            {
                this.Text += selectedItem.Text + "; ";
            }
        }
    }
}